@@ -6,7 +6,7 @@ from typing import (
     AsyncIterator,
     BinaryIO,
     Callable,
-    Dict,
+    Mapping,
     Optional,
     Union,
 )
@@ -60,6 +60,8 @@ class RetryConfiguration:
 
 
 def _make_log_before_function(s: str) -> Callable[[tenacity.RetryCallState], None]:
+    """Create a function used to log before a retry attempt."""
+
     def log(retry_state: tenacity.RetryCallState) -> None:
 
         if retry_state.attempt_number > 1:
@@ -73,6 +75,8 @@ def _make_log_before_function(s: str) -> Callable[[tenacity.RetryCallState], Non
 def _make_log_before_sleep_function(
     s: str,
 ) -> Callable[[tenacity.RetryCallState], None]:
+    """Create a function used when a call made through tenacity fails."""
+
     def log(retry_state: tenacity.RetryCallState) -> None:
         if (retry_state.next_action is not None) and (retry_state.outcome is not None):
             duration = retry_state.next_action.sleep
@@ -88,13 +92,25 @@ def _make_log_before_sleep_function(
     return log
 
 
+def _make_retrying(s: str, config: RetryConfiguration) -> tenacity.AsyncRetrying:
+    """Create a tenacity retry object."""
+
+    return tenacity.AsyncRetrying(
+        retry=tenacity.retry_if_exception_type(aiohttp.ClientError),
+        stop=tenacity.stop_after_attempt(config.retry_attempts),
+        wait=tenacity.wait_exponential(max=config.max_retry_period_seconds),
+        before=_make_log_before_function(s),
+        before_sleep=_make_log_before_sleep_function(s),
+    )
+
+
 async def upload(
     endpoint: Union[str, yarl.URL],
     file: BinaryIO,
     metadata: Optional[common.Metadata] = None,
     client_session: Optional[aiohttp.ClientSession] = None,
     config: RetryConfiguration = RetryConfiguration(),
-    headers: Optional[Dict[str, str]] = None,
+    headers: Optional[Mapping[str, str]] = None,
 ) -> Optional[yarl.URL]:
     """Upload a file to a tus server.
 
@@ -114,21 +130,8 @@ async def upload(
     if metadata is None:
         metadata = {}
 
-    retrying_create = tenacity.AsyncRetrying(
-        retry=tenacity.retry_if_exception_type(aiohttp.ClientError),
-        stop=tenacity.stop_after_attempt(config.retry_attempts),
-        wait=tenacity.wait_exponential(max=config.max_retry_period_seconds),
-        before=_make_log_before_function("upload creation"),
-        before_sleep=_make_log_before_sleep_function("upload creation"),
-    )
-
-    retrying_upload_file = tenacity.AsyncRetrying(
-        retry=tenacity.retry_if_exception_type(aiohttp.ClientError),
-        stop=tenacity.stop_after_attempt(config.retry_attempts),
-        wait=tenacity.wait_exponential(max=config.max_retry_period_seconds),
-        before=_make_log_before_function("upload"),
-        before_sleep=_make_log_before_sleep_function("upload"),
-    )
+    retrying_create = _make_retrying("upload creation", config)
+    retrying_upload_file = _make_retrying("upload", config)
 
     try:
         ctx: Union[aiohttp.ClientSession, AsyncContextManager[aiohttp.ClientSession]]
@@ -177,7 +180,7 @@ async def metadata(
     endpoint: Union[str, yarl.URL],
     client_session: Optional[aiohttp.ClientSession] = None,
     config: RetryConfiguration = RetryConfiguration(),
-    headers: Optional[Dict[str, str]] = None,
+    headers: Optional[Mapping[str, str]] = None,
 ) -> Optional[common.Metadata]:
     """Retrieve the metadata of an upload.
 
@@ -195,13 +198,7 @@ async def metadata(
     else:
         url = endpoint
 
-    retrying_metadata = tenacity.AsyncRetrying(
-        retry=tenacity.retry_if_exception_type(aiohttp.ClientError),
-        stop=tenacity.stop_after_attempt(config.retry_attempts),
-        wait=tenacity.wait_exponential(max=config.max_retry_period_seconds),
-        before=_make_log_before_function("query metadata"),
-        before_sleep=_make_log_before_sleep_function("query metadata"),
-    )
+    retrying_metadata = _make_retrying("query metadata", config)
 
     try:
         ctx: Union[aiohttp.ClientSession, AsyncContextManager[aiohttp.ClientSession]]
